@@ -1,7 +1,9 @@
+from http import client
 from pathlib import Path
 from python_on_whales import DockerClient
 from jsonschema import validate
 import collections.abc
+from typing import Union
 import uuid
 import os
 import re
@@ -18,10 +20,15 @@ from flowui.scripts.create_docker_compose_file import create_docker_compose_file
 from flowui.scripts.run_operator_docker import run_operator as run_operator_in_docker
 from flowui.scripts.run_operator_bash import run_operator as run_operator_in_bash
 from flowui.scripts.load_operator import load_operator_models_from_path
+from flowui.client.github_rest_client import GithubRestClient
 from flowui.schemas.operator_metadata import OperatorMetadata
 from flowui.utils.metadata_default import metadata_default
+from flowui.exceptions.exceptions import (
+    MissingOperatorRepositoryFileError, 
+    MissingEnvVarError, 
+    NoMatchingDependencyForOperatorError
+)
 from flowui.logger import get_configured_logger
-from typing import Union
 
 
 def dict_deep_update(d, u):
@@ -33,6 +40,13 @@ def dict_deep_update(d, u):
     return d
 
 
+class SetEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, set):
+            return list(obj)
+        return json.JSONEncoder.default(self, obj)
+
+
 class CommandLineInterface:
 
     def __init__(self):
@@ -78,14 +92,15 @@ class CommandLineInterface:
         )
 
         action_choices = [
-            'organize', 
+            'pre-run',
             'run', 
             'stop', 
             'create-compose-file', 
+            'create-operators-repository',
+            'organize', 
             'run-local-dev', 
             'run-operator-docker', 
             'run-operator-bash', 
-            'create-operators-repository'
         ]
         parser.add_argument(
             'action', 
@@ -119,16 +134,167 @@ class CommandLineInterface:
         return args
     
 
+    def pre_run(self) -> None:
+        # Create local folder to mount for Airflow home directory
+        self.volume_path_airflow_host = Path.cwd() / "airflow"
+        if not self.volume_path_airflow_host.is_dir():
+            self.volume_path_airflow_host.mkdir(parents=True, exist_ok=True)
+            (self.volume_path_airflow_host / "logs").mkdir(parents=True, exist_ok=True)
+            (self.volume_path_airflow_host / "plugins").mkdir(parents=True, exist_ok=True)
+            (self.volume_path_airflow_host / "dags").mkdir(parents=True, exist_ok=True)
+        
+        # Create local folder to mount for Workspaces directories
+        self.volume_path_workspaces_host = Path.cwd() / "workspaces"
+        if not self.volume_path_workspaces_host.is_dir():
+            self.volume_path_workspaces_host.mkdir(parents=True, exist_ok=True)
+
+        # Create configuration file
+        data = {
+            "flowui": {
+                "FLOWUI_DEPLOY_MODE": "local-bash",
+                "VOLUME_PATH_AIRFLOW_HOST": str(self.volume_path_airflow_host),
+                "VOLUME_PATH_WORKSPACES_HOST": str(self.volume_path_workspaces_host),
+                "OPERATORS_REPOSITORY_PATH_HOST": "",
+                "OPERATORS_REPOSITORIES_NAMES_GITHUB": [
+                    "Tauffer-Consulting/flowui_example_project"
+                ]
+            } 
+        }
+        with open('config.toml', 'wb') as out:
+            tomli_w.dump(data, out)
+
+
+    # def _create_volume_folders(self):
+    #     """Function to create required folders in user mount path
+    #     Required folders are:
+    #     - airflow
+    #     ----|logs
+    #     ----|plugins
+    #     ----| dags 
+    #     - operators_repository
+    #     --| operators
+    #     --| workflows
+    #     - datasets
+    #     - runs
+    #     - tmp
+    #     """
+    #     self.logger.info("Creating required folders in mount...")
+
+    #     # Operators repository folders
+    #     operators_repository_path = base_path / "operators_repository"
+    #     if not operators_repository_path.exists():
+    #         operators_repository_path.mkdir(parents=True, exist_ok=True)
+    #     operators_repository_operators_path = operators_repository_path / "operators"
+    #     if not operators_repository_operators_path.exists():
+    #         operators_repository_operators_path.mkdir(parents=True, exist_ok=True)
+    #     operators_repository_workflows_path = operators_repository_path / "workflows"
+    #     if not operators_repository_workflows_path.exists():
+    #         operators_repository_workflows_path.mkdir(parents=True, exist_ok=True)
+
+    #     # Datasets folder
+    #     datasets_path = base_path / "datasets"
+    #     if not datasets_path.exists():
+    #         datasets_path.mkdir(parents=True, exist_ok=True)
+
+    #     # Runs folder
+    #     runs_path = base_path / "runs"
+    #     if not runs_path.exists():
+    #         runs_path.mkdir(parents=True, exist_ok=True)
+
+    #     # Temp folder
+    #     tmp_path = base_path / "tmp"
+    #     if not tmp_path.exists():
+    #         tmp_path.mkdir(parents=True, exist_ok=True)
+        # self.logger.info("Required folders created successfully!")
+
+
     def run(self) -> None:
         """
         Run FlowUI using local deploy scope.
         This will create a docker-compose.yaml file and run it.
         """
-        self._validate_repository_structure()
-        self._create_mnt_required_folders()
-        self.create_docker_compose_file()
-        self.docker = DockerClient(compose_files=["docker-compose.yaml"])
-        self.docker.compose.up()
+        # Load FlowUI's running instance config.toml
+        if Path("config.toml").is_file():
+            with open("config.toml", "rb") as f:
+                flowui_config_dict = tomli.load(f)
+        else:
+            raise FileNotFoundError()
+        
+        # Load metadata from remote Operators repositories
+        if not os.getenv("GITHUB_ACCESS_TOKEN", None):
+            raise MissingEnvVarError(missing_vars=["GITHUB_ACCESS_TOKEN"])
+        client = GithubRestClient()
+
+        missing_secrets = list()
+        secrets_dict = dict()
+        for repo in flowui_config_dict["flowui"]["OPERATORS_REPOSITORIES_NAMES_GITHUB"]:
+            # Load compiled metadata
+            try:
+                r = client.get_contents(
+                    repo_name=repo,
+                    file_path=".flowui/compiled_metadata.json"
+                )
+                compiled_metadata = json.loads(r.decoded_content)
+            except:
+                raise MissingOperatorRepositoryFileError(missing_file=".flowui/compiled_metadata.json", operators_repository=repo)
+
+            # Load dependencies map
+            try:
+                r = client.get_contents(
+                    repo_name=repo,
+                    file_path=".flowui/dependencies_map.json"
+                )
+                dependencies_map = json.loads(r.decoded_content)
+            except:
+                raise MissingOperatorRepositoryFileError(missing_file=".flowui/dependencies_map.json", operators_repository=repo)
+
+            # Load repository's config
+            r = client.get_contents(
+                repo_name=repo,
+                file_path="config.toml"
+            )
+            repo_config = tomli.loads(r.decoded_content.decode("utf-8"))
+            repo_name = repo_config["repository"]["REPOSITORY_NAME"]
+            
+            # Check if list of secrets defined by Operators repositories are presenti in host's ENV
+            for k, v in compiled_metadata.items():
+                secrets_schema = v.get("secrets_schema", None)
+                dep_name = self._get_dependency_for_operator(
+                    operator_name=k, 
+                    dependecies_map=dependencies_map, 
+                    repo_name=repo_name
+                )
+                if secrets_schema:
+                    for sn in secrets_schema["properties"].keys():
+                        full_secret_name = f"{repo_name}_{sn}"
+                        if not os.getenv(full_secret_name, None):
+                            missing_secrets.append(f"{full_secret_name} used by {k}")
+
+        if len(missing_secrets) > 0:
+            raise MissingEnvVarError(missing_vars=missing_secrets)
+
+        self.create_docker_compose_file(
+            run_scope="deploy-local",
+            flowui_config=flowui_config_dict
+        )
+
+        # Run docker compose
+        # self.docker = DockerClient(compose_files=["docker-compose.yaml"])
+        # self.docker.compose.up()
+
+
+    def _get_dependency_for_operator(self, operator_name: str, dependecies_map: dict, repo_name: str):
+        """
+        _summary_
+
+        Args:
+            operator_name (str): _description_
+            dependecies_map (dict): _description_
+        """
+        for k, v in dependecies_map.items():
+            if operator_name in v["operators"]:
+                return k
+        raise NoMatchingDependencyForOperatorError(operator_name=operator_name, repo_name=repo_name)
 
 
     def run_local_dev(self) -> None:
@@ -136,10 +302,6 @@ class CommandLineInterface:
         Run FlowUI using local development environment scope.
         This will create a docker-compose.yaml file and run it.
         """ 
-        self._validate_repository_structure()
-        if not os.environ.get('FLOWUI_PATH_HOST'):
-            os.environ['FLOWUI_PATH_HOST'] = str(Path('../flowui').resolve())
-        self._create_mnt_required_folders()
         self.create_docker_compose_file(run_scope="local-dev")
         self.docker = DockerClient(compose_files=["docker-compose.yaml"])
         self.docker.compose.up()
@@ -364,74 +526,12 @@ class CommandLineInterface:
         self.logger.info("docker-compose.yaml file created successfully!")
 
 
-    def _create_mnt_required_folders(self):
-        """Function to create required folders in user mount path
-        Required folders are:
-        - airflow
-        ----|logs
-        ----|plugins
-        ----| dags 
-        - operators_repository
-        --| operators
-        --| workflows
-        - datasets
-        - runs
-        - tmp
-        """
-        self.logger.info("Creating required folders in mount...")
-
-        # Mount folder
-        base_path = Path(self.config_required_fields.get("VOLUME_MOUNT_PATH_HOST"))
-        if not base_path.exists():
-            base_path.mkdir(parents=True, exist_ok=True)
-
-        # Airflow folders
-        airflow_path = base_path / "airflow"
-        if not airflow_path.exists():
-            airflow_path.mkdir(parents=True, exist_ok=True)
-        airflow_logs_path = airflow_path / "logs"
-        if not airflow_logs_path.exists():
-            airflow_logs_path.mkdir(parents=True, exist_ok=True)
-        airflow_plugins_path = airflow_path / "plugins"
-        if not airflow_plugins_path.exists():
-            airflow_plugins_path.mkdir(parents=True, exist_ok=True)
-        airflow_dags_path = airflow_path / "dags"
-        if not airflow_dags_path.exists():
-            airflow_dags_path.mkdir(parents=True, exist_ok=True)
-
-        # Operators repository folders
-        operators_repository_path = base_path / "operators_repository"
-        if not operators_repository_path.exists():
-            operators_repository_path.mkdir(parents=True, exist_ok=True)
-        operators_repository_operators_path = operators_repository_path / "operators"
-        if not operators_repository_operators_path.exists():
-            operators_repository_operators_path.mkdir(parents=True, exist_ok=True)
-        operators_repository_workflows_path = operators_repository_path / "workflows"
-        if not operators_repository_workflows_path.exists():
-            operators_repository_workflows_path.mkdir(parents=True, exist_ok=True)
-
-        # Datasets folder
-        datasets_path = base_path / "datasets"
-        if not datasets_path.exists():
-            datasets_path.mkdir(parents=True, exist_ok=True)
-
-        # Runs folder
-        runs_path = base_path / "runs"
-        if not runs_path.exists():
-            runs_path.mkdir(parents=True, exist_ok=True)
-
-        # Temp folder
-        tmp_path = base_path / "tmp"
-        if not tmp_path.exists():
-            tmp_path.mkdir(parents=True, exist_ok=True)
-        self.logger.info("Required folders created successfully!")
-
-
-    def _create_dependencies_map(self, save_map_as_file: bool = True) -> None:
+    def _create_dependencies_map(self, repo_name: str, save_map_as_file: bool = True) -> None:
         """
         Construct a map between Operators and unique definitions for docker images dependencies
 
         Args:
+            repo_name (str): Name of the repository, as defined in config.toml
             save_map_as_file (bool, optional): Set if dependencies_map will be saved as file. Defaults to True.
 
         Raises:
@@ -442,11 +542,18 @@ class CommandLineInterface:
     
         operators_images_map = {}
         for op_i, (operator_name, operator_metadata) in enumerate(compiled_metadata.items()):
+            
+            if operator_metadata.get("secrets_schema"):
+                operator_secrets = set(operator_metadata.get("secrets_schema")["properties"].keys())
+            else:
+                operator_secrets = set()
+
             if op_i == 0:
                 operators_images_map = {
-                    "flowui-dependency-0": {
+                    f"{repo_name}_flowui_dependency_0": {
                         "dependency": operator_metadata["dependency"],
-                        "operators": [operator_name]
+                        "operators": [operator_name],
+                        "secrets": operator_secrets
                     }
                 }
             else:
@@ -456,12 +563,14 @@ class CommandLineInterface:
                 for i, dep_key in enumerate(existing_keys):
                     if all([operator_metadata["dependency"][k] == operators_images_map[dep_key]["dependency"][k] for k in operator_metadata["dependency"].keys()]):
                         operators_images_map[dep_key]["operators"].append(operator_name)
+                        operators_images_map[dep_key]["secrets"].update(operator_secrets)
                         skip_new_image = True
                         continue
                 if not skip_new_image:
-                    operators_images_map[f"flowui-dependency-{len(existing_keys)}"] = {
+                    operators_images_map[f"{repo_name}_flowui_dependency_{len(existing_keys)}"] = {
                         "dependency": operator_metadata["dependency"],
-                        "operators": [operator_name]
+                        "operators": [operator_name],
+                        "secrets": operator_secrets
                     }
 
         if not operators_images_map:
@@ -470,7 +579,7 @@ class CommandLineInterface:
         if save_map_as_file:
             map_file_path = self.organized_flowui_path / "dependencies_map.json"
             with open(map_file_path, "w") as outfile:
-                json.dump(operators_images_map, outfile, indent=4)
+                json.dump(operators_images_map, outfile, indent=4, cls=SetEncoder)
 
 
     def _create_compiled_operators_metadata(self) -> None:  
@@ -514,30 +623,30 @@ class CommandLineInterface:
         6. Create example dependency files
         7. Run flowui organize
         """
-        repository_name = input("Enter the repository's name: ") or f"new-repository-{str(uuid.uuid4())[0:8]}"
+        repository_name = input("Enter the repository's name: ") or f"new_repository_{str(uuid.uuid4())[0:8]}"
         while not self._validate_repository_name(repository_name):
-            repository_name = input("\nInvalid repository name. Should have only numbers, letters, underscores and dashes. \nEnter a new repository name: ") or f"new-repository-{str(uuid.uuid4())[0:8]}"
+            repository_name = input("\nInvalid repository name. Should have only numbers, letters and underscores. \nEnter a new repository name: ") or f"new-repository-{str(uuid.uuid4())[0:8]}"
         cwd = Path.cwd()
         repository_folder = cwd / repository_name
         if repository_folder.is_dir():
             raise Exception("Repository folder already exists")
         repository_folder.mkdir()
         self.logger.info(f"Creating repository in {repository_folder}")
-        self._create_config_toml(repository_path=repository_folder, repository_name=repository_name)
+        self._create_config_toml_repository(repository_path=repository_folder, repository_name=repository_name)
         self._create_repository_folders(repository_folder)
         subprocess.call(f"cd {str(repository_folder)} && flowui organize", shell=True)
 
 
     @staticmethod
     def _validate_repository_name(name: str):
-        regex = r'^[A-Za-z0-9_-]*$'
+        regex = r'^[A-Za-z0-9_]*$'
         pattern = re.compile(regex)
         if pattern.match(name):
             return True
         return False
 
 
-    def _create_config_toml(self, repository_path: Path, repository_name: str) -> None:
+    def _create_config_toml_repository(self, repository_path: Path, repository_name: str) -> None:
         """
         Create Operators repository basic config.toml file
 
@@ -592,12 +701,20 @@ class CommandLineInterface:
         # Validate repository
         self._validate_repository_structure()
         self._validate_operators_repository()
+
+        # Load config
+        with open("config.toml", "rb") as f:
+            repo_config = tomli.load(f)
+        repo_name = repo_config["repository"]["REPOSITORY_NAME"]
         
         # Create compiled metadata from Operators metadata.json files and add data input schema
         self._create_compiled_operators_metadata()
         
         # Generate dependencies_map.json file
-        self._create_dependencies_map(save_map_as_file=True)
+        self._create_dependencies_map(
+            repo_name=repo_name,
+            save_map_as_file=True
+        )
 
         # Build and publish the images
         if self.args.build_images:
@@ -624,6 +741,7 @@ def cli():
     cli_args = cli.args
     cli_func_map = {        
         # Run locally
+        "pre-run": cli.pre_run,
         "run": cli.run,
         "stop": cli.stop_compose,
         "create-compose-file": cli.create_docker_compose_file,
