@@ -3,8 +3,8 @@ from typing import List, Dict
 
 
 class Dependency(BaseModel):
-    docker_base_image: str = Field(
-        description="Docker image to be used as base for the container that will run this Operator", 
+    docker_image: str = Field(
+        description="Docker image to be used to run this Operator", 
         example="python:3.9-slim", 
         default=None
     )
@@ -18,6 +18,7 @@ class Dependency(BaseModel):
         example="requirements.txt",
         default=None
     )
+    # TODO - validation: one and only one of these options
 
 
 class InputSchema(BaseModel):
@@ -27,18 +28,21 @@ class InputSchema(BaseModel):
     properties: Dict
 
 
+class OutputSchema(BaseModel):
+    title: str
+    description: str
+    type: str
+    properties: Dict
+
+
 class OperatorMetadata(BaseModel):
     name: str = Field(
         description="Operator name", 
         example="ExampleOperator", 
         # regex=    # TODO - regex for *Operator  
     )
-    version: str = Field(
-        description="Operator version", 
-        example="0.1.0", 
-        # regex=    # TODO - regex for 0.1.0 type
-    )
     dependency: Dependency = None
     tags: List[str] = None
     style: Dict = None # TODO - add model for the style dictionary
-    input_schema: InputSchema = None
\ No newline at end of file
+    input_schema: InputSchema = None
+    output_schema: OutputSchema = None
\ No newline at end of file
