@@ -1,7 +1,6 @@
 from pathlib import Path
 import importlib
 import sys
-import json
 
 
 def load_operator_class_from_path(operators_folder_path: str, operator_name: str, operator_metadata: dict):
@@ -30,5 +29,6 @@ def load_operator_models_from_path(operators_folder_path: str, operator_name: st
     operator_model_module = importlib.import_module(f"{operator_name}.models")
     operator_input_model_class = getattr(operator_model_module, "InputModel")
     operator_output_model_class = getattr(operator_model_module, "OutputModel")
+    operator_secrets_model_class = getattr(operator_model_module, "SecretsModel", None)
 
-    return operator_input_model_class, operator_output_model_class
\ No newline at end of file
+    return operator_input_model_class, operator_output_model_class, operator_secrets_model_class
\ No newline at end of file
